import click
import os
from sys import platform
import sys
import time
import pyfiglet
import re
from PyInquirer import prompt


class MultiCommand(click.Group):
    def command(self, *args, **kwargs):
        """Behaves the same as `click.Group.command()` except if passed
        a list of names, all after the first will be aliases for the first.
        """
        def decorator(f):
            if isinstance(args[0], list):
                _args = [args[0][0]] + list(args[1:])
                for alias in args[0][1:]:
                    cmd = super(MultiCommand, self).command(
                        alias, *args[1:], **kwargs)(f)
                    cmd.short_help = "Alias for '{}'".format(_args[0])
            else:
                _args = args
            cmd = super(MultiCommand, self).command(
                *_args, **kwargs)(f)
            return cmd
        return decorator


def command_process_step(start_text, command):
    print("⌛ "+start_text+"\n")
    print(f">> {command}\n")
    os.system(command)


def clear_screen():
    if platform == "linux" or platform == "linux2" or platform == "darwin":
        os.system("clear")
    elif platform == "win32":
        os.system("cls")


def process_ok(finished_process):
    clear_screen()
    text = pyfiglet.figlet_format("flaskmng", font="slant")
    print(text)
    for i in finished_process:
        print("✔ "+i)


def process_step(start_text, func):
    print("⌛ "+start_text+"\n")
    func()


def create_folder(folder_name):
    def wrapper():
        os.mkdir(folder_name)
    return wrapper


def in_virtualenv():
    def get_base_prefix_compat():
        return getattr(sys, "base_prefix", None) or getattr(sys, "real_prefix", None) or sys.prefix
    return get_base_prefix_compat() != sys.prefix


def supports_color():
    plat = sys.platform
    supported_platform = plat != 'Pocket PC' and (plat != 'win32' or
                                                  'ANSICON' in os.environ)
    is_a_tty = hasattr(sys.stdout, 'isatty') and sys.stdout.isatty()
    return supported_platform and is_a_tty


def success_message(text):
    print("🎉 "+text)


def info_message(text):
    print("ℹ️ "+" "+text)


def make_compatible(name):
    result = ""
    for ch in name:
        if ch in [" ", "-"]:
            result += "_"
            continue
        elif re.match(r'^[0-9]$', str(ch)):
            continue
        result += ch
    return result


def hl(name):
    if supports_color():
        return f"\033[94m{name}\033[0m"
    return f"\"{name}\""


def take_input(text):
    questions = [
        {
            'type':'input',
            'name':'data',
            'message':text
        }
    ]

    answers = prompt(questions)
    return answers['data']


import sys
import click

from .startproject import startproject
from .startapp import startapp

commands = [
    startproject,
    startapp
]


main = click.CommandCollection(sources=commands)

if __name__ == '__main__':
    args = sys.argv
    if "--help" in args or len(args) == 1:
        print("flaskmng")
    try:
        main()
    except Exception as e:
        print("❌ "+str(e))



from os.path import join
import click
from python_script_manager.package import PSMReader
from ..utils import (
    MultiCommand,
    command_process_step,
    info_message,
    process_ok,
    process_step,
    create_folder,
    in_virtualenv,
    success_message,
    make_compatible,
    hl,
    take_input
)
from .utils import (
    create_app_init_py,
    create_models_py,
    create_forms_py,
    create_routes_py,
    append_app_datas,
)


@click.group(cls=MultiCommand)
def startapp():
    pass


@startapp.command("startapp")
def startapp_command():
    if not in_virtualenv():
        raise Exception("You must be in virtual environment to run")
    
    processes = []
    process_ok(processes)

    # Taking app name
    app_name = take_input("Enter name for app:")

    # Creating app folder
    prj_name = PSMReader().get_config().get('PROJECT_NAME', None)
    if not prj_name :
        raise Exception("You deleted your project name from psm.json file. Please add it.")
    
    process_step(f"Creating {hl(join(prj_name, app_name))} folder...",
                create_folder(join(prj_name, app_name)))
    processes.append(f"Created {hl(join(prj_name, app_name))} folder")
    process_ok(processes)

    # Creating __init__.py
    process_step(f"Creating {hl(join(prj_name, app_name, '__init__.py'))}...",
                create_app_init_py(join(prj_name, app_name)))
    processes.append(f"Created {hl(join(prj_name, app_name, '__init__.py'))}")
    process_ok(processes)

    # Creating templates folder
    process_step(f"Creating {hl(join(prj_name, app_name, 'templates'))} folder...",
                create_folder(join(prj_name, app_name, 'templates')))
    processes.append(f"Created {hl(join(prj_name, app_name, 'templates'))} folder")
    process_ok(processes)
    
    # Creating models.py
    process_step(f"Creating {hl(join(prj_name, app_name, 'models.py'))}...",
                create_models_py(join(prj_name, app_name)))
    processes.append(f"Created {hl(join(prj_name, app_name, 'models.py'))}")
    process_ok(processes)

    # Creating forms.py
    process_step(f"Creating {hl(join(prj_name, app_name, 'forms.py'))}...",
                create_forms_py(join(prj_name, app_name)))
    processes.append(f"Created {hl(join(prj_name, app_name, 'forms.py'))}")
    process_ok(processes)

    # Creating routes.py
    process_step(f"Creating {hl(join(prj_name, app_name, 'routes.py'))}...",
                create_routes_py(prj_name,app_name))
    processes.append(f"Created {hl(join(prj_name, app_name, 'routes.py'))}")
    process_ok(processes)

    # Appending app datas to __init__.py
    process_step(f"Appending {app_name} modules to {hl(join(prj_name, '__init__.py'))}...",
                append_app_datas(prj_name, app_name))
    processes.append(f"Appended {app_name} modules to {hl(join(prj_name, '__init__.py'))}")
    process_ok(processes)

    # Appending app name to psm config
    psm = PSMReader()
    psm_config = psm.get_config()
    psm_config['APPS'].append(app_name)
    psm.set_config(psm_config)
    psm.write()

    # Output success message
    success_message(f"Successfully created {hl(app_name)}")
    info_message(f"Use {hl('psm run')} to run your application")
